@@ -18,14 +18,19 @@ eval 'exec /usr2/local/bin/perl -S $0 ${1+"$@"}'
 package Math::Currency;
 
 use strict;
+use utf8;
+use version;
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $PACKAGE $FORMAT $LC_MONETARY
   $accuracy $precision $div_scale $round_mode $use_int $always_init);
 use Exporter;
 use Math::BigFloat 1.60;
 use overload '""' => \&bstr;
 use POSIX qw(locale_h);
+use Encode::Locale;
+use Encode ();
 
-use constant _LEGACY_MATH_BIGINT => (Math::BigInt->VERSION <= '1.999717');
+use constant _LEGACY_MATH_BIGINT => 
+    (version->parse(Math::BigInt->VERSION) <= version->parse('1.999717'));
 
 @ISA = qw(Exporter Math::BigFloat);
 
@@ -41,7 +46,7 @@ use constant _LEGACY_MATH_BIGINT => (Math::BigInt->VERSION <= '1.999717');
   Money
 );
 
-$VERSION = 0.48;
+$VERSION = 0.49;
 
 $PACKAGE = __PACKAGE__;
 
@@ -338,6 +343,20 @@ sub localize    #08/17/02 7:58:PM
 
     my $localeconv = POSIX::localeconv(); 
 
+    # localeconv()'s character encoding depends on the current locale setting,
+    # so it is necessary to decode the currency symbol.
+    Encode::Locale::reinit(); # neede in case locale was changed with setlocale()
+
+    for my $key (keys %$localeconv) {
+        # POSIX::localeconv() changed behaviour between 5.20 and 5.22.  As of
+        # 5.22, it sets the UTF-8 flag for localeconv() returned data if hte
+        # data is in UTF-8 format, so if POSIX::localeconv() already turned on
+        # the UTF-8 flag, we should not decode the data.
+        unless (utf8::is_utf8($$localeconv{$key})) {
+            $$localeconv{$key} = Encode::decode(locale => $$localeconv{$key});
+        }
+    }
+
     # so you can test to see if locale was effective
     return 0 if ! exists $localeconv->{'currency_symbol'};
 
@@ -369,6 +388,26 @@ sub localize    #08/17/02 7:58:PM
     return 1;
 }
 
+{
+    my $locales;
+    sub available_locales {
+        unless (defined $locales) {
+            $locales = [];
+
+            open my $fh, '-|', 'locale -a' or die $!;
+
+            while (my $locale = <$fh>) {
+                chomp $locale;
+                push @$locales, $locale;
+            }
+
+            close $fh;
+        }
+
+        return @$locales;
+    }
+}
+
 ############################################################################
 sub unknown_currency    #02/03/05 4:37am
 ############################################################################
@@ -376,33 +415,37 @@ sub unknown_currency    #02/03/05 4:37am
 {
     my ($currency) = @_;
     $DB::single=1;
-    open LOCALES, "-|", "locale -a";
-    while (my $LOCALE = <LOCALES>) {
-        chomp($LOCALE);
+
+    # remember current locale
+    my $original_locale = setlocale( LC_ALL );
+
+    # we need to save a copy of $VERSION here becuase the effective locale can
+    # render $VERSION as X,YY instead of Y.YY for exmaple
+    my $version = "$VERSION";
+
+    for my $LOCALE (available_locales()) {
         setlocale( LC_ALL, $LOCALE );
         my $localeconv = POSIX::localeconv();
-	if ( $LOCALE eq $currency || 
-	    (defined $localeconv->{'int_curr_symbol'}
-             and $localeconv->{'int_curr_symbol'} =~ /$currency/ )
-	   )
+        if ( $LOCALE eq $currency or
+            ($localeconv->{int_curr_symbol} || '') =~ /$currency/ )
         {
             my $format = \$LC_MONETARY->{$currency};
             Math::Currency->localize($format);
-	    (my $int_curr = $$format->{'INT_CURR_SYMBOL'}) =~  s/ //g;
-	    $LC_MONETARY->{$int_curr} = $LC_MONETARY->{$currency}
-		unless exists $LC_MONETARY->{$int_curr};	
-	    eval <<"EOP";
+            (my $int_curr = $$format->{INT_CURR_SYMBOL}) =~  s/ //g;
+            $LC_MONETARY->{$int_curr} = $LC_MONETARY->{$currency}
+                unless exists $LC_MONETARY->{$int_curr};
+            eval <<"EOP";
 package Math::Currency::${LOCALE};
 use vars qw(\$VERSION \@ISA \$LANG);
 
-\$VERSION = $Math::Currency::VERSION;
+\$VERSION = $version;
 \$LANG  = '$LOCALE';
 \@ISA = qw/Math::Currency/;
 1;
 package Math::Currency::${int_curr};
 use vars qw(\$VERSION \@ISA \$LANG);
 
-\$VERSION = $Math::Currency::VERSION;
+\$VERSION = $version;
 \$LANG  = '$LOCALE';
 \@ISA = qw/Math::Currency/;
 1;
@@ -410,7 +453,9 @@ EOP
             last;
         }
     }
-    close LOCALES;
+
+    # restore the original locale
+    setlocale( LC_ALL, $original_locale );
 }
 
 # additional methods needed to get/set package globals
