@@ -1,17 +1,18 @@
 #!/usr/bin/perl -w
 package Math::Currency::en_GB;
 
+use utf8;
 use Exporter;
 use Math::Currency qw($LC_MONETARY $FORMAT);
 use vars qw($VERSION @ISA $LANG);
 
-$VERSION = 0.48;
+$VERSION = 0.49;
 $LANG  = 'en_GB';
 @ISA     = qw(Exporter Math::Currency);
 
 $LC_MONETARY->{en_GB} = {
 	INT_CURR_SYMBOL	=>	'GBP ',
-	CURRENCY_SYMBOL	=>	'',
+	CURRENCY_SYMBOL	=>	'£',
 	MON_DECIMAL_POINT	=>	'.',
 	MON_THOUSANDS_SEP	=>	',',
 	MON_GROUPING	=>	'3',
